/****************************************************************/
/* FILENAME: main.c                            		        */
/* DESCRIPTION: This program uses the MSP430F449 to write 256 	*/
/*    samples to the DAC8564/5.                                 */
/*    The samples are generated by sine/cosine functions        */
/*                                                              */
/* Author: Jojo Parguian                                        */
/* Precision Analog Applications, Dallas, TX                    */
/* CREATED 06/18/08(C) BY TEXAS INSTRUMENTS INCORPORATED.  	*/
/* VERSION: 01   						*/
/****************************************************************/
#include <msp430x44x.h>
#include <math.h>

/**************** Function prototypes ***********************************/
/*                                                                      */
void init_sys(void);                  /* MSP430 Initialisation routine  */
void setupClock(int);
void delay(int);                      /* Software delay                 */
void dac_convert(int, int);           /* Do the dac conversion          */
void InitTables(int);
void int_ref_disable(int, int, int);  /* Disable the Internal Reference */
void int_ref_enable(void);            /* Enable the Internal Reference  */
void int_ref_ON (void);               /* Enable the Internal Reference  */
                                      /* all the time                   */

/************************************************************************/
/*                                                                      */
/* HPA449 variable declarations                                         */
/*                                                                      */
/************************************************************************/
#define DAC_CS       0x20  //p3.5
#define LDAC         0x40  //p3.6
#define Resonator (0)
#define disable_ref (0)
#define enable_ref (0)

/************************************************************************/
/*                                                                      */
/* main() variable declarations                                         */
/*                                                                      */
/************************************************************************/
#define BLOCK_SZ         256          /* size of data buffer */
#define TABLE_SIZE       256

int sinetable[TABLE_SIZE], costable[TABLE_SIZE], byte0, byte1, byte2;
int i, y;

/******************************************************************************\
* Function: InitTables()
* Description: Initializes Sine and Cosine Values
\******************************************************************************/
void main(void)
{
  int  val0, val1, val2;
  int  dac_A, dac_B, dac_C, dac_D;
  setupClock(Resonator);
  init_sys();                   	// Initialise the MSP430
  InitTables(1);                        // Number defines the cycles per period

      if (disable_ref)
          {
            val0 = 0x00;
            val1 = 0x20;
            val2 = 0x01;

            int_ref_disable(val0, val1, val2);
          }
      else if (enable_ref)
      {
        int_ref_enable();
      }
      else
      {
        int_ref_ON();
      }


      do {
          dac_A = 0x0;
          dac_B = 0x2;
          dac_C = 0x4;
          dac_D = 0x26;

           for(i=0; i<BLOCK_SZ; i++)
           {
            dac_convert(dac_A, sinetable[i]);  // Send sine values to the DAC A = sinetable[i]
            dac_convert(dac_B, sinetable[i]);  // Send sine values to the DAC B = sinetable[i]
            dac_convert(dac_C, sinetable[i]);  // Send sine values to the DAC C = sinetable[i]
            dac_convert(dac_D, sinetable[i]);  // Send sine values to the DAC D = sinetable[i]
           }
         }
       while (1);              // Endless Loop
} /*end of main*/			      	

/************************************************************/
/* Prototype - init_table                                   */
/*                                                          */
/*  Description                                             */
/*  This prototype initialises the waveform table           */
/************************************************************/
void InitTables(int cycles)
{
	int y;
	for (y = 0; y < TABLE_SIZE; y++)
	{
	  sinetable[y] = (unsigned int)((sin(2 * 3.141593659 * cycles * y / TABLE_SIZE) + 1.0) * 32767);
	  costable[y] = (unsigned int)((cos(2 * 3.141593659 * cycles * y / TABLE_SIZE) + 1.0) * 32767);
	}
}

/************************************************************/
/* Prototype - init_sys                                     */
/*                                                          */
/*  Description                                             */
/*  This prototype initialises the MSP430F449               */
/************************************************************/
void init_sys(void)
{
void setupports(void);  /* Local function prototype         */
void setupSPI(void);    /* Local function prototype         */

setupports();
setupSPI();
//  _EINT();                              // Enable interrupts
}

/************************************************************/
/* Prototype - setupClock                                     */
/*                                                          */
/*  Description                                             */
/*  This prototype sets-up the XT2 oscillator and tests     */
/*  that it has settled before moving on                    */
/************************************************************/
void setupClock (int RES)
{
    switch (RES)
    {
        case  0:
          { WDTCTL = WDTPW + WDTHOLD;
            /* D=2, N=121, no mod */
            SCFQCTL=121;
            // SCFI0 = 0100 0100
            SCFI0=0x44;
            // DCOPLUS=1, XTS_FLL=0
            // XCAPXPF=00
            FLL_CTL0=0x80;
            // SMCLK=0, XT2OFF=1, SELMX=00, SELS=0, FLL_DIVX=10
            // FLL_CTL1= 0010 0010
            FLL_CTL1=0x22;
          }
          break;
        case  1:
          { WDTCTL = WDTPW + WDTHOLD;           // Stop watchdog timer
            FLL_CTL0&=XT2OFF;                    // Switch on the XT2 osc.
            FLL_CTL1|=SELM_XT2+SELS;                // Select XT2 osc. for
            // Test the osc. flag bit
            do {
                IFG1 &= ~OFIFG;            // Clear the OFIFG bit
                } while (OFIFG&IFG1);      //
          }
          break;
    }
}

/************************************************************/
/* Prototype - setupports                                   */
/*  Description                                             */
/*  This prototype sets-up the GPIO ports as appropriate    */
/************************************************************/
void setupports (void)
{
// SPI port for DAC
P3SEL = BIT3 + BIT2 + BIT1;                 // Bits 3, 2 & 1 are assigned as SPI specific pins
P3DIR = DAC_CS+LDAC;                        // Set the DAC_CS bit as an output
P3OUT = DAC_CS+LDAC;                        // De-assert DAC_CS for the DAC - HIGH
P4SEL = BIT5 + BIT3 + BIT2;

//SetupInterrupts
P1IFG |= 0x00;
P1IES |= 0x40;
P1IE |= 0x40;
}

/************************************************************/
/* Prototype - setupSPI                                     */
/*                                                          */
/*  Description                                             */
/*  This prototype sets-up the P3 for communication via SPI */
/************************************************************/
void setupSPI (void)

/************************************************************/
/* System definitions                                       */
/************************************************************/
#define     SPI0en       0x40
#define     SPI1en       0x10

{
	

ME1 |= USPIE0;                      // Module Enable - SPI1
U0CTL &= ~SWRST;                    // Make sure the RESET bit is off
U0CTL |= CHAR + SYNC + MM;          // USART0 module operation
                                    // CHAR = 1 => 8-bit data
                                    // SYNC = 1 => SPI selected
                                    // MM = 1 => master mode,
U0TCTL |= CKPH + CKPL + SSEL0 + SSEL1 + STC;      // USART0 Tranmit control register
U0BR0  = 0x02;                      // Divide SMCLK by 2 => transfer clock
U0BR1  = 0x00;                      //
U0MCTL = 0x00;                      // Modulation control - not used. Ensure
                                    // all these bits are reset
}

/************************************************************/
/* Prototype - delay                                        */
/*  Description                                             */
/*  This prototype gives a delay of around 1 second         */
/************************************************************/

void delay(int time)
      {
      unsigned int i;
      for (i = 0; i < time; i++);
      }
/************************************************************/
/* Prototype - int_ref_disable                              */
/*  Description                                             */
/*  This prototype disables the DAC internal reference      */
/************************************************************/
void int_ref_disable (val0, val1, val2)
{
  byte0 = val0;
  byte1 = val1;
  byte2 = val2;

  P3OUT &= ~(DAC_CS);              // Set DAC /CS low
  while ((IFG1 & UTXIFG0) == 0);
        U0TXBUF = byte2 ;          // Send clocks to the DAC, this shifts CMD byte
	                           // Wait until all 8 bits have been received.
  while ((IFG1 & UTXIFG0) == 0);
        U0TXBUF = byte1;           // Send clocks to the DAC, this shifts MSB byte
	                           // Wait until all 8 bits have been received.
  while ((IFG1 & UTXIFG0) == 0);
        U0TXBUF = byte0;           // Send clocks to the DAC, this shifts LSB byte
	                           // Wait until all 8 bits have been received.
  delay(3);	                   // Wait until all 8 bits have been received.
  P3OUT |= (DAC_CS);               // Set DAC /CS high
}

/************************************************************/
/* Prototype - int_ref_enable                               */
/*  Description                                             */
/*  This prototype enables the DAC internal reference       */
/************************************************************/
void int_ref_enable (void)
{
  byte0 = 0x00;
  byte1 = 0x00;
  byte2 = 0x01;

  P3OUT &= ~(DAC_CS);                // Set DAC /CS low for 1st write cycle
  while ((IFG1 & UTXIFG0) == 0);
        U0TXBUF = byte2 ;            // Send clocks to the DAC, this shifts CMD byte
	                             // Wait until all 8 bits have been received.
  while ((IFG1 & UTXIFG0) == 0);
        U0TXBUF = byte1;             // Send clocks to the DAC, this shifts MSB byte
	                             // Wait until all 8 bits have been received.
  while ((IFG1 & UTXIFG0) == 0);
        U0TXBUF = byte0;             // Send clocks to the DAC, this shifts LSB byte
	                             // Wait until all 8 bits have been received.
  delay(3);	                     // Wait until all 8 bits have been received.
  P3OUT |= (DAC_CS);                 // Set DAC /CS high
}

/*******************************************************************/
/* Prototype - int_ref_ON                                          */
/*  Description                                                    */
/*  This prototype enables the DAC internal reference all the time */
/*******************************************************************/
void int_ref_ON (void)
{
  byte0 = 0x00;
  byte1 = 0x10;
  byte2 = 0x01;

  P3OUT &= ~(DAC_CS);                // Set DAC /CS low for 2nd write cycle
  while ((IFG1 & UTXIFG0) == 0);
        U0TXBUF = byte2 ;            // Send clocks to the DAC, this shifts CMD byte
	                             // Wait until all 8 bits have been received.
  while ((IFG1 & UTXIFG0) == 0);
        U0TXBUF = byte1;             // Send clocks to the DAC, this shifts MSB byte
	                             // Wait until all 8 bits have been received.
  while ((IFG1 & UTXIFG0) == 0);
        U0TXBUF = byte0;             // Send clocks to the DAC, this shifts LSB byte
	                             // Wait until all 8 bits have been received.
  delay(3);	                     // Wait until all 8 bits have been received.
  P3OUT |= (DAC_CS);                 // Set DAC /CS high
}

/************************************************************/
/* Prototype - convert                                      */
/*  Description                                             */
/*  This prototype does the DAC conversion                  */
/************************************************************/
void dac_convert (dac_Ch, value)

{
byte0 = value >>8;
byte1 = value >>0;
byte2 = dac_Ch;

P3OUT &= ~(DAC_CS);                // Set DAC /CS low
while ((IFG1 & UTXIFG0) == 0);
      U0TXBUF = byte2 ;            // Send clocks to the DAC, this shifts CMD byte
	                           // Wait until all 8 bits have been received.
while ((IFG1 & UTXIFG0) == 0);
      U0TXBUF = byte0;             // Send clocks to the DAC, this shifts MSB byte
	                           // Wait until all 8 bits have been received.
while ((IFG1 & UTXIFG0) == 0);
      U0TXBUF = byte1 ;            // Send clocks to the DAC, this shifts LSB byte
	                           // Wait until all 8 bits have been received.

delay(1);	                   // Wait until all 8 bits have been received.
P3OUT |= (DAC_CS);                 // Set DAC /CS high

}
